/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "outputbuild.h"

OutputBuild::OutputBuild()
{
  swin = new Gtk::ScrolledWindow();
  build_box = new Gtk::TreeView();
  
  swin->add(*build_box);
  swin->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  build_box->set_headers_visible(false);
}

Gtk::TreeView* OutputBuild::get_build_box()
{ return build_box;
}

Gtk::ScrolledWindow* OutputBuild::get_swin()
{ return swin;
}

// Run the specified command with pipes. This function receives the command to
// run, the TreeView object and the environment settings class object. It 
// cleans the build output and runs the command.
//
// This function has the ability to highlight warnings, errors and other compiler
// messages with the user-specified colors as well as check if the build is able
// to find Makefiles
bool OutputBuild::run_command(Glib::ustring command, Gtk::TreeView *build_box, EnvironmentSettings *env)
{
  char buff[32];
  FILE *in;
  bool successful = true, autogenerate = false;
  int location;
  Glib::ustring text = "\0", line;
  
  erase();
  add_row(command.substr(0, command.find("2>&1", 0)), "#000000");
  build_box->get_selection()->select(build_box->get_model()->get_iter("0"));
    
  // Open pipe & loop until the program is complete
  in = popen(command.c_str(), "r");
  
  while (fgets(buff, sizeof(buff), in) && continue_build) 
  {
    text += buff;
    // Find the endl's & split them to be added to build_box
    location = text.find("\n",0);
    while (location != Glib::ustring::npos) 
    {
      // Check to see if autogen.sh must be run before build
      if (text.find("No targets specified",0) != string::npos)
        autogenerate = true;
        
      // Append line & erase it from the buffer
      line = text.substr(0,location);
      text.erase(0,location+1);
      location = text.find("\n",0);

      // Apply Error or Warning colors
      if (line.find("error: ") != Glib::ustring::npos) 
      {
        add_row(line, env->get_error_color());
        successful = false;
      } 
      else if (line.find(": ") != Glib::ustring::npos)
        add_row(line, env->get_warning_color());
      else
        add_row(line, "#000000");
      
      // Update the build_box when there is output
      update_output();
    }
  }
    
  // Close the stream, & print whether command was successful or not
  pclose(in);
  if (successful)
    add_row("Completed ... Successful", "#000000");
  else
    add_row("Completed ... Failed", "#000000");
  build_box->scroll_to_row(build_model->get_path(row));
  
  // Return false if the user must run autogen.sh before building & then do it
  if (autogenerate)
    return false;
  
  return true;
}

void OutputBuild::erase()
{
  // Clean up build_box & get it ready to accept output
  build_box->remove_all_columns();
  build_model = Gtk::ListStore::create(build_columns);
  build_box->set_model(build_model);
  
  // Append the columns & create the cell renderer
  build_box->append_column("Compiler Output", build_columns.col_text);
  color_column = build_box->get_column(0);
  Gtk::CellRenderer *cell_renderer = color_column->get_first_cell_renderer();
  renderer = dynamic_cast<Gtk::CellRendererText*>(cell_renderer);
  color_column->add_attribute(renderer->property_foreground(), build_columns.col_color);
  
  continue_build = true;
}

void OutputBuild::add_row(Glib::ustring text, Glib::ustring color)
{
  // Add the command to the box
  row = *(build_model->append());
  row[build_columns.col_text] = text;
  row[build_columns.col_color] = color;
}

void OutputBuild::update_output()
{
  while(Gtk::Main::instance()->events_pending()) 
  {
    Gtk::Main::instance()->iteration();
    build_box->scroll_to_row(build_model->get_path(row));
  }
}

void OutputBuild::stop_build()
{ continue_build = false;
}
